/*
 * Name: Geolocation
 * Developer: Amberlyn Schjoll, Maria Paolini, Alex Wallace
 * Date: 5/21/2019
 */
define(function () {
    var install = function (holderElement, context) {

        var row = context.getRow();
        var latitudeField = row.getCellByColumnName('Latitude');
        var longitudeField = row.getCellByColumnName('Longitude');
        var refreshRate = parseInt(context.getParameter("RefreshRate"), 10) || -1;


        if (!navigator.geolocation) {
            console.log("Sorry, your browser does not support geolocation services.");
            return;
        }
        
        var $panelContainer = $(holderElement).parents(".vinyl-panel-container");

        if ($panelContainer.data("geolocation")) {
            console.log("Ignore another install");
            return;
        }

        var intervalnumber = null;
        
        // Checks if the panel still exists in the DOM.
        // The first cell this is installed on can come and go, but the panel should be the one
        // determining if we should stop this operation.
        function shouldContinue() {
            if (!document.contains($panelContainer.get(0))) {
                console.log("Stop refresh, panel not in dom");
                clearInterval(intervalnumber);
                return false;
            }
            
            return true;
        }

        function initGeolocation() {
            if (!shouldContinue()) {
               return; 
            }

            // Call getCurrentPosition with success and failure callbacks
            navigator.geolocation.getCurrentPosition(success, fail);
        }

        function success(position) {
            if (!shouldContinue()) {
                return;
            }
            
            var currentLatitude = position.coords.latitude.toFixed(4);
            var currentLongitude = position.coords.longitude.toFixed(4);

            var storedLatitude = parseFloat(latitudeField.value).toFixed(4);
            var storedLongitude = parseFloat(longitudeField.value).toFixed(4);

            if (currentLatitude === storedLatitude && currentLongitude === storedLongitude) {
                console.log("GPS coordinates unchanged");
                return;
            }

            console.log("Saving Latitude:Longitude ", currentLatitude, currentLongitude);

            if (!context.isEditState()) {
                // May cause new install() run
                row.edit();
            }

            latitudeField.value = currentLatitude;
            longitudeField.value = currentLongitude;

            // Definitely will cause new install() run
            row.save();
        }

        function fail() {
            // Could not obtain location
            console.log("Sorry, geolocation failed.");
            clearInterval(intervalnumber);
        }

        // Always run once
        setTimeout(initGeolocation, 0);
        
        if (refreshRate > 0) {
            // If using refresh rate, run repeatedly
            console.log("Start monitoring GPS location. Refreshing every ", refreshRate, "ms");
            intervalnumber = setInterval(initGeolocation, refreshRate);
        }

        $panelContainer.data("geolocation", 1);
    };
    
    var uninstall = function (holderElement, context) {
        // Nothing to uninstall.
    };

    return {
        callbacks: {
            events: {
                // adding and removing
                install: install,
                uninstall: uninstall
            }
        }
    };
});
